@@ -9,6 +9,7 @@ from epimodels import BaseModel
 import logging
 from collections import OrderedDict
 from functools import lru_cache
+import numba
 
 logging.basicConfig(filename='epimodels.log', filemode='w', level=logging.DEBUG)
 
@@ -32,6 +33,15 @@ class ContinuousModel(BaseModel):
         #     logging.Error('Invalid model type: {}'.format(model_type))
 
     def __call__(self, inits: list, trange: list, totpop: float, params: dict, method: str = 'RK45', **kwargs):
+        """
+        Run the model
+        :param inits: initial contitions
+        :param trange: time range: [t0, tf]
+        :param totpop: total population size
+        :param params: dictionary of parameters
+        :param method: integration method. default is 'RK45'
+        :param kwargs: Additional parameters passed on to solve_ivp
+        """
         self.method = method
         self.kwargs = kwargs
         sol = self.run(inits, trange, totpop, params, **kwargs)
@@ -39,7 +49,7 @@ class ContinuousModel(BaseModel):
         res['time'] = sol.t
         self.traces.update(res)
 
-    def model(self, t: float, y: list, params: list):
+    def _model(self, t: float, y: list, params: list):
         raise NotImplementedError
 
     @property
@@ -49,7 +59,7 @@ class ContinuousModel(BaseModel):
     def run(self, inits, trange, totpop, params, **kwargs):
         # model = model_types[self.model_type]['function']
         params['N'] = totpop
-        sol = solve_ivp(lambda t, y: self.model(t, y, params), trange, inits, self.method, **self.kwargs)
+        sol = solve_ivp(lambda t, y: self._model(t, y, params), trange, inits, self.method, **kwargs)
         return sol
 
 
@@ -57,15 +67,16 @@ class SIR(ContinuousModel):
     def __init__(self):
         super().__init__()
         self.state_variables = OrderedDict({'S': 'Susceptible', 'I': 'Infectious', 'R': 'Removed'})
-        self.parameters = OrderedDict({'beta': r'\beta', 'gamma': r'\gamma'})
+        self.parameters = OrderedDict({'beta': r'$\beta$', 'gamma': r'$\gamma$'})
         self.model_type = 'SIR'
 
-    def model(self, t: float, y: list, params: dict) -> list:
+    @numba.jit
+    def _model(self, t: float, y: list, params: dict) -> list:
         """
         SIR Model.
-        :param t:
-        :param y:
-        :param params:
+        :param t: time step
+        :param y: state of the model at time t
+        :param params: parameter dictionary
         :return:
         """
         S, I, R = y
@@ -77,6 +88,28 @@ class SIR(ContinuousModel):
         ]
 
 
+class SIR1D(ContinuousModel):
+    def __init__(self):
+        super().__init__()
+        self.state_variables = OrderedDict({'R': 'Recovered'})
+        self.parameters = {'R0': r'{\cal R}_0', 'gamma': r'\gamma', 'S0': r'S_0'}
+        self.model_type = 'SIR1D'
+
+    def _model(self, t: float, y: list, params: dict) -> list:
+        """
+        One dimensional SIR model
+        :param t:
+        :param y:
+        :param params:
+        """
+        N = params['N']
+        R = y
+        R0, gamma, S0 = params['R0'], params['gamma'], params['S0']
+        return [
+            gamma * (N - R - (S0 * np.exp(-R0 * R)))
+        ]
+
+
 class SIS(ContinuousModel):
     def __init__(self):
         super().__init__()
@@ -85,7 +118,8 @@ class SIS(ContinuousModel):
         self.model_type = 'SIS'
 
     # @lru_cache(1000)
-    def model(self, t: float, y: list, params: dict) -> list:
+    @numba.jit
+    def _model(self, t: float, y: list, params: dict) -> list:
         """
         SIS Model.
         :param t:
@@ -108,7 +142,8 @@ class SIRS(ContinuousModel):
         self.parameters = OrderedDict({'beta': r'$\beta$', 'gamma': r'$\gamma$', 'xi': r'$\xi$'})
         self.model_type = 'SIRS'
 
-    def model(self, t: float, y: list, params: dict) -> list:
+    @numba.jit
+    def _model(self, t: float, y: list, params: dict) -> list:
         """
         SIR Model.
         :param t:
@@ -132,7 +167,8 @@ class SEIR(ContinuousModel):
         self.parameters = OrderedDict({'beta': r'$\beta$', 'gamma': r'$\gamma$', 'epsilon': r'$\epsilon$'})
         self.model_type = 'SEIR'
 
-    def model(self, t: float, y: list, params: dict) -> list:
+    @numba.jit
+    def _model(self, t: float, y: list, params: dict) -> list:
         S, E, I, R = y
         beta, gamma, epsilon, N = params['beta'], params['gamma'], params['epsilon'], params['N']
         return [
@@ -149,14 +185,15 @@ class SEQIAHR(ContinuousModel):
         self.state_variables = OrderedDict(
             {'S': 'Susceptible', 'E': 'Exposed', 'I': 'Infectious', 'A': 'Asymptomatic', 'H': 'Hospitalized',
              'R': 'Removed', 'C': 'Cumulative hospitalizations', 'D': 'Cumulative deaths'})
-        self.parameters = OrderedDict({'chi': r'$\chi', 'phi': r'$\phi$', 'beta': r'$\beta$',
+        self.parameters = OrderedDict({'chi': r'$\chi$', 'phi': r'$\phi$', 'beta': r'$\beta$',
                                        'rho': r'$\rho$', 'delta': r'$\delta$', 'gamma': r'$\gamma$',
                                        'alpha': r'$\alpha$', 'mu': r'$\mu$',
                                        'p': '$p$', 'q': '$q$', 'r': '$r$'
                                        })
         self.model_type = 'SEQIAHR'
 
-    def model(self, t: float, y: list, params: dict) -> list:
+    @numba.jit
+    def _model(self, t: float, y: list, params: dict) -> list:
         S, E, I, A, H, R, C, D = y
         chi, phi, beta, rho, delta, gamma, alpha, mu, p, q, r, N = params.values()
         lamb = beta * (I + A)
